@@ -28,7 +28,6 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include <arpa/inet.h>
-#include <stdlib.h>
 #include <sys/socket.h>
 
 #include <config.h>
@@ -46,6 +45,7 @@ mqtt3_context *mqtt3_context_init(int sock)
 	
 	context->connected = false;
 	context->disconnecting = false;
+	context->duplicate = false;
 	context->sock = sock;
 	context->last_msg_in = time(NULL);
 	context->last_msg_out = time(NULL);
@@ -54,7 +54,7 @@ mqtt3_context *mqtt3_context_init(int sock)
 	context->id = NULL;
 
 	context->in_packet.payload = NULL;
-	mqtt3_context_packet_cleanup(&context->in_packet);
+	_mosquitto_packet_cleanup(&context->in_packet);
 	context->out_packet = NULL;
 
 	addrlen = sizeof(addr);
@@ -81,39 +81,26 @@ mqtt3_context *mqtt3_context_init(int sock)
  */
 void mqtt3_context_cleanup(mqtt3_context *context)
 {
+	struct _mosquitto_packet *packet;
 	if(!context) return;
 
 	if(context->sock != -1){
 		mqtt3_socket_close(context);
 	}
-	if(context->clean_session){
+	if(context->clean_session && !context->duplicate){
 		mqtt3_db_subs_clean_session(context->id);
 		mqtt3_db_messages_delete(context->id);
 		mqtt3_db_client_delete(context);
 	}
 	if(context->address) mqtt3_free(context->address);
 	if(context->id) mqtt3_free(context->id);
-	mqtt3_context_packet_cleanup(&(context->in_packet));
+	_mosquitto_packet_cleanup(&(context->in_packet));
 	while(context->out_packet){
-		mqtt3_context_packet_cleanup(context->out_packet);
+		_mosquitto_packet_cleanup(context->out_packet);
+		packet = context->out_packet;
 		context->out_packet = context->out_packet->next;
+		mqtt3_free(packet);
 	}
 	mqtt3_free(context);
 }
 
-void mqtt3_context_packet_cleanup(struct _mqtt3_packet *packet)
-{
-	if(!packet) return;
-
-	/* Free data and reset values */
-	packet->command = 0;
-	packet->have_remaining = 0;
-	packet->remaining_count = 0;
-	packet->remaining_mult = 1;
-	packet->remaining_length = 0;
-	if(packet->payload) mqtt3_free(packet->payload);
-	packet->payload = NULL;
-	packet->to_process = 0;
-	packet->pos = 0;
-}
-
