@@ -105,7 +105,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <sqlite3.h>
 #include <stdio.h>
 #include <string.h>
-#include <unistd.h>
 
 #include <config.h>
 #include <mqtt3.h>
@@ -136,11 +135,6 @@ static int _mqtt3_db_upgrade_1_2(void);
 static int _mqtt3_db_transaction_rollback(void);
 #endif
 
-#ifdef WITH_CLIENT
-/* Client callback for publish events - this WILL change. */
-int (*client_publish_callback)(const char *, int, uint32_t, const uint8_t *, int) = NULL;
-#endif
-
 int mqtt3_db_open(mqtt3_config *config)
 {
 #ifdef WITH_REGEX
@@ -246,6 +240,8 @@ int mqtt3_db_close(void)
 
 	sqlite3_shutdown();
 
+	if(db_filepath) mqtt3_free(db_filepath);
+
 	return 0;
 }
 
@@ -420,7 +416,7 @@ static int _mqtt3_db_upgrade(void)
 {
 	int rc = 0;
 	sqlite3_stmt *stmt = NULL;
-	int version;
+	int version = 0;
 
 	if(!db) return 1;
 
@@ -657,7 +653,11 @@ int mqtt3_db_client_insert(mqtt3_context *context, int will, int will_retain, in
 		}else if(oldsock != context->sock){
 			/* Client is already connected, disconnect old version */
 			mqtt3_log_printf(MQTT3_LOG_ERR, "Client %s already connected, closing old connection.", context->id);
+#ifdef WITH_BROKER
+			mqtt3_context_close_duplicate(oldsock);
+#else
 			close(oldsock);
+#endif
 		}
 		mqtt3_db_client_update(context, will, will_retain, will_qos, will_topic, will_message);
 	}else{
@@ -756,11 +756,12 @@ int mqtt3_db_client_will_queue(mqtt3_context *context)
 	int qos;
 	const uint8_t *payload;
 	int retain;
+	int will;
 
 	if(!context || !context->id) return 1;
 
 	if(!stmt){
-		stmt = _mqtt3_db_statement_prepare("SELECT will_topic,will_qos,will_message,will_retain FROM clients WHERE id=?");
+		stmt = _mqtt3_db_statement_prepare("SELECT will,will_topic,will_qos,will_message,will_retain FROM clients WHERE id=?");
 		if(!stmt){
 			return 1;
 		}
@@ -768,10 +769,16 @@ int mqtt3_db_client_will_queue(mqtt3_context *context)
 	if(sqlite3_bind_text(stmt, 1, context->id, strlen(context->id), SQLITE_STATIC) != SQLITE_OK) rc = 1;
 	dbrc = sqlite3_step(stmt);
 	if(dbrc == SQLITE_ROW){
-		topic = (const char *)sqlite3_column_text(stmt, 0);
-		qos = sqlite3_column_int(stmt, 1);
-		payload = sqlite3_column_text(stmt, 2);
-		retain = sqlite3_column_int(stmt, 3);
+		will = sqlite3_column_int(stmt, 0);
+		if(!will){
+			sqlite3_reset(stmt);
+			sqlite3_clear_bindings(stmt);
+			return 0;
+		}
+		topic = (const char *)sqlite3_column_text(stmt, 1);
+		qos = sqlite3_column_int(stmt, 2);
+		payload = sqlite3_column_text(stmt, 3);
+		retain = sqlite3_column_int(stmt, 4);
 		if(!rc){
 			if(mqtt3_db_messages_easy_queue(context->id, topic, qos, strlen((const char *)payload), payload, retain)) rc = 1;
 		}
@@ -1000,7 +1007,7 @@ int mqtt3_db_message_count(int *count)
 	return rc;
 }
 
-int mqtt3_db_message_delete(const char *client_id, uint16_t mid, mqtt3_msg_direction dir)
+int mqtt3_db_message_delete(const char *client_id, uint16_t mid, enum mosquitto_msg_direction dir)
 {
 	int rc = 0;
 	static sqlite3_stmt *stmt = NULL;
@@ -1042,7 +1049,7 @@ int mqtt3_db_message_delete_by_oid(int64_t oid)
 	return rc;
 }
 
-int mqtt3_db_message_insert(const char *client_id, uint16_t mid, mqtt3_msg_direction dir, mqtt3_msg_status status, int qos, int64_t store_id)
+int mqtt3_db_message_insert(const char *client_id, uint16_t mid, enum mosquitto_msg_direction dir, enum mqtt3_msg_status status, int qos, int64_t store_id)
 {
 	/* Warning: Don't start transaction in this function. */
 	int rc = 0;
@@ -1102,7 +1109,7 @@ int mqtt3_db_message_insert(const char *client_id, uint16_t mid, mqtt3_msg_direc
 	return rc;
 }
 
-int mqtt3_db_message_update(const char *client_id, uint16_t mid, mqtt3_msg_direction dir, mqtt3_msg_status status)
+int mqtt3_db_message_update(const char *client_id, uint16_t mid, enum mosquitto_msg_direction dir, enum mqtt3_msg_status status)
 {
 	/* Warning: Don't start transaction in this function. */
 	int rc = 0;
@@ -1171,41 +1178,17 @@ int mqtt3_db_messages_queue(const char *source_id, const char *topic, int qos, i
 	uint8_t client_qos;
 	uint8_t msg_qos;
 	uint16_t mid;
-#ifdef WITH_CLIENT
-	static sqlite3_stmt *stmt = NULL;
-	uint32_t payloadlen;
-	const uint8_t *payload = NULL;
-#endif
 
 	/* Find all clients that subscribe to topic and put messages into the db for them. */
 	if(!source_id || !topic || !store_id) return 1;
 
-#ifdef WITH_CLIENT
-	if(client_publish_callback){
-		if(!stmt){
-			stmt = _mqtt3_db_statement_prepare("SELECT payloadlen,payload FROM message_store WHERE id=?");
-			if(!stmt){
-				return 1;
-			}
-		}
-		if(sqlite3_bind_int64(stmt, 1, store_id) != SQLITE_OK) rc = 1;
-		if(!rc && sqlite3_step(stmt) == SQLITE_ROW){
-			payloadlen = sqlite3_column_int(stmt, 0);
-			if(payloadlen){
-				payload = sqlite3_column_blob(stmt, 1);
-			}
-
-			client_publish_callback(topic, qos, payloadlen, payload, retain);
-		}
-		sqlite3_reset(stmt);
-		sqlite3_clear_bindings(stmt);
-	}
-#endif
 	if(retain){
 		if(mqtt3_db_retain_insert(topic, store_id)) rc = 1;
 	}
-	if(!mqtt3_db_sub_search_start(source_id, topic)){
-		while(!mqtt3_db_sub_search_next(&client_id, &client_qos)){
+	if(!mqtt3_db_sub_search_start(source_id, topic, qos)){
+		while(sqlite3_step(stmt_sub_search) == SQLITE_ROW){
+			client_id = (char *)sqlite3_column_text(stmt_sub_search, 0);
+			client_qos = sqlite3_column_int(stmt_sub_search, 1);
 			if(qos > client_qos){
 				msg_qos = client_qos;
 			}else{
@@ -1218,18 +1201,19 @@ int mqtt3_db_messages_queue(const char *source_id, const char *topic, int qos, i
 			}
 			switch(msg_qos){
 				case 0:
-					if(mqtt3_db_message_insert(client_id, mid, md_out, ms_publish, msg_qos, store_id)) rc = 1;
+					if(mqtt3_db_message_insert(client_id, mid, mosq_md_out, ms_publish, msg_qos, store_id)) rc = 1;
 					break;
 				case 1:
-					if(mqtt3_db_message_insert(client_id, mid, md_out, ms_publish_puback, msg_qos, store_id)) rc = 1;
+					if(mqtt3_db_message_insert(client_id, mid, mosq_md_out, ms_publish_puback, msg_qos, store_id)) rc = 1;
 					break;
 				case 2:
-					if(mqtt3_db_message_insert(client_id, mid, md_out, ms_publish_pubrec, msg_qos, store_id)) rc = 1;
+					if(mqtt3_db_message_insert(client_id, mid, mosq_md_out, ms_publish_pubrec, msg_qos, store_id)) rc = 1;
 					break;
 			}
-			if(client_id) mqtt3_free(client_id);
 		}
 	}
+	sqlite3_reset(stmt_sub_search);
+	sqlite3_clear_bindings(stmt_sub_search);
 	return rc;
 }
 
@@ -1276,7 +1260,7 @@ int mqtt3_db_message_timeout_check(unsigned int timeout)
 	int64_t OID;
 	int status;
 	int retries;
-	mqtt3_msg_status new_status = ms_invalid;
+	enum mqtt3_msg_status new_status = ms_invalid;
 
 	if(!stmt_select){
 		stmt_select = _mqtt3_db_statement_prepare("SELECT OID,status,retries FROM messages WHERE timestamp<?");
@@ -1325,7 +1309,7 @@ int mqtt3_db_message_timeout_check(unsigned int timeout)
 	return 0;
 }
 
-int mqtt3_db_message_release(const char *client_id, uint16_t mid, mqtt3_msg_direction dir)
+int mqtt3_db_message_release(const char *client_id, uint16_t mid, enum mosquitto_msg_direction dir)
 {
 	int rc = 0;
 	static sqlite3_stmt *stmt = NULL;
@@ -1421,25 +1405,25 @@ int mqtt3_db_message_write(mqtt3_context *context)
 
 				case ms_publish_puback:
 					if(!mqtt3_raw_publish(context, retries, qos, retain, mid, topic, payloadlen, payload)){
-						mqtt3_db_message_update(context->id, mid, md_out, ms_wait_puback);
+						mqtt3_db_message_update(context->id, mid, mosq_md_out, ms_wait_puback);
 					}
 					break;
 
 				case ms_publish_pubrec:
 					if(!mqtt3_raw_publish(context, retries, qos, retain, mid, topic, payloadlen, payload)){
-						mqtt3_db_message_update(context->id, mid, md_out, ms_wait_pubrec);
+						mqtt3_db_message_update(context->id, mid, mosq_md_out, ms_wait_pubrec);
 					}
 					break;
 				
 				case ms_resend_pubrel:
 					if(!mqtt3_raw_pubrel(context, mid)){
-						mqtt3_db_message_update(context->id, mid, md_out, ms_wait_pubrel);
+						mqtt3_db_message_update(context->id, mid, mosq_md_out, ms_wait_pubrel);
 					}
 					break;
 
 				case ms_resend_pubcomp:
 					if(!mqtt3_raw_pubcomp(context, mid)){
-						mqtt3_db_message_update(context->id, mid, md_out, ms_wait_pubcomp);
+						mqtt3_db_message_update(context->id, mid, mosq_md_out, ms_wait_pubcomp);
 					}
 					break;
 			}
@@ -1525,16 +1509,22 @@ static int _mqtt3_db_regex_create(const char *topic, char **regex)
 	int pos;
 	int i;
 	int sys = 0;
+	char *start_slash;
 
 	if(!topic || !regex) return 1;
 
-	local_topic = mqtt3_strdup(topic);
-	if(!local_topic) return 1;
-
-	if(!strncmp(local_topic, "$SYS", 4)){
+	if(!strncmp(topic, "$SYS", 4)){
 		sys = 1;
 	}
 	hier = 0;
+	if(topic[0] == '/'){
+		local_topic = mqtt3_strdup(topic+1);
+		start_slash = "/";
+	}else{
+		local_topic = mqtt3_strdup(topic);
+		start_slash = "";
+	}
+	if(!local_topic) return 1;
 	stmp = local_topic;
 	while(stmp){
 		stmp = index(stmp, '/');
@@ -1560,7 +1550,7 @@ static int _mqtt3_db_regex_create(const char *topic, char **regex)
 	if(hier > 1){
 		token = strtok(local_topic, "/");
 		if(!sys){
-			pos = sprintf(local_regex, "^(?:(?:(?:\\Q%s\\E|\\+)(?!$))", token);
+			pos = sprintf(local_regex, "^%s(?:(?:(?:\\Q%s\\E|\\+)(?!$))", start_slash, token);
 		}else{
 			pos = sprintf(local_regex, "^(?:(?:(?:\\Q%s\\E)(?!$))", token);
 		}
@@ -1585,7 +1575,7 @@ static int _mqtt3_db_regex_create(const char *topic, char **regex)
 		}
 	}else{
 		if(!sys){
-			pos = sprintf(local_regex, "^(?:(?:(?:\\Q%s\\E|\\+))|#)$", local_topic);
+			pos = sprintf(local_regex, "^%s(?:(?:(?:\\Q%s\\E|\\+))|#)$", start_slash, local_topic);
 		}else{
 			pos = sprintf(local_regex, "^(?:(?:(?:%s|\\+)))$", local_topic);
 		}
@@ -1610,13 +1600,18 @@ static int _mqtt3_db_retain_regex_create(const char *sub, char **regex)
 	if(!sub || !regex) return 1;
 
 	if(strncmp(sub, "$SYS", 4)){
-		sys = "^(?!\\$SYS)";
+		if(sub[0] == '/'){
+			sys = "^/(?!\\$SYS)";
+			local_sub = mqtt3_strdup(sub+1);
+		}else{
+			sys = "^(?!\\$SYS)";
+			local_sub = mqtt3_strdup(sub);
+		}
 	}else{
 		sys = "^";
+		local_sub = mqtt3_strdup(sub);
 	}
-	local_sub = mqtt3_strdup(sub);
 	if(!local_sub) return 1;
-
 	hier = 0;
 	stmp = local_sub;
 	while(stmp){
@@ -1734,7 +1729,7 @@ int mqtt3_db_retain_delete(const char *topic)
 int mqtt3_db_retain_queue(mqtt3_context *context, const char *sub, int sub_qos)
 {
 	int rc = 0;
-	static sqlite3_stmt *stmt = NULL;;
+	static sqlite3_stmt *stmt = NULL;
 #ifdef WITH_REGEX
 	char *regex = NULL;
 #endif
@@ -1776,13 +1771,13 @@ int mqtt3_db_retain_queue(mqtt3_context *context, const char *sub, int sub_qos)
 		}
 		switch(qos){
 			case 0:
-				if(mqtt3_db_message_insert(context->id, mid, md_out, ms_publish, qos, store_id)) rc = 1;
+				if(mqtt3_db_message_insert(context->id, mid, mosq_md_out, ms_publish, qos, store_id)) rc = 1;
 				break;
 			case 1:
-				if(mqtt3_db_message_insert(context->id, mid, md_out, ms_publish_puback, qos, store_id)) rc = 1;
+				if(mqtt3_db_message_insert(context->id, mid, mosq_md_out, ms_publish_puback, qos, store_id)) rc = 1;
 				break;
 			case 2:
-				if(mqtt3_db_message_insert(context->id, mid, md_out, ms_publish_pubrec, qos, store_id)) rc = 1;
+				if(mqtt3_db_message_insert(context->id, mid, mosq_md_out, ms_publish_pubrec, qos, store_id)) rc = 1;
 				break;
 		}
 	}
@@ -1793,7 +1788,7 @@ int mqtt3_db_retain_queue(mqtt3_context *context, const char *sub, int sub_qos)
 
 int mqtt3_db_store_clean(void)
 {
-	static sqlite3_stmt *stmt = NULL;;
+	static sqlite3_stmt *stmt = NULL;
 	int rc = 0;
 
 	if(!stmt){
@@ -1901,7 +1896,7 @@ int mqtt3_db_sub_delete(const char *client_id, const char *sub)
  * Will use regex for pattern matching if compiled with WITH_REGEX defined.
  * Wildcards in subscriptions are disabled if WITH_REGEX not defined.
  */
-int mqtt3_db_sub_search_start(const char *source_id, const char *topic)
+int mqtt3_db_sub_search_start(const char *source_id, const char *topic, int qos)
 {
 	/* Warning: Don't start transaction in this function. */
 	int rc = 0;
@@ -1918,21 +1913,19 @@ int mqtt3_db_sub_search_start(const char *source_id, const char *topic)
 		stmt_sub_search = _mqtt3_db_statement_prepare("SELECT client_id,qos FROM subs "
 				"JOIN clients ON subs.client_id=clients.id "
 				"WHERE regexp(?, subs.sub)"
-				" AND ((clients.sock=-1 AND subs.qos<>0) OR clients.sock<>-1)"
+				" AND ((clients.sock=-1 AND subs.qos<>0 AND ?<>0) OR clients.sock<>-1)"
 				" AND (clients.is_bridge=0 OR (clients.is_bridge=1 AND clients.id<>?))");
 #else
 		stmt_sub_search = _mqtt3_db_statement_prepare("SELECT client_id,qos FROM subs "
 				"JOIN clients ON subs.client_id=clients.id "
 				"WHERE subs.sub=?"
-				" AND ((clients.sock=-1 AND subs.qos<>0) OR clients.sock<>-1)"
+				" AND ((clients.sock=-1 AND subs.qos<>0 AND ?<>0) OR clients.sock<>-1)"
 				" AND (clients.is_bridge=0 OR (clients.is_bridge=1 AND clients.id<>?))");
 #endif
 		if(!stmt_sub_search){
 			return 1;
 		}
 	}
-	sqlite3_reset(stmt_sub_search);
-	sqlite3_clear_bindings(stmt_sub_search);
 
 #ifdef WITH_REGEX
 	if(_mqtt3_db_regex_create(topic, &regex)) return 1;
@@ -1940,30 +1933,12 @@ int mqtt3_db_sub_search_start(const char *source_id, const char *topic)
 #else
 	if(sqlite3_bind_text(stmt_sub_search, 1, topic, strlen(topic), SQLITE_STATIC) != SQLITE_OK) rc = 1;
 #endif
-	if(sqlite3_bind_text(stmt_sub_search, 2, source_id, strlen(source_id), SQLITE_STATIC) != SQLITE_OK) rc = 1;
+	if(sqlite3_bind_int(stmt_sub_search, 2, qos) != SQLITE_OK) rc = 1;
+	if(sqlite3_bind_text(stmt_sub_search, 3, source_id, strlen(source_id), SQLITE_STATIC) != SQLITE_OK) rc = 1;
 
 	return rc;
 }
 
-/* Retrieve next matching subscription from search.
- * mqtt3_db_sub_search_start() must have been called prior to calling this function.
- * Returns matching client id and qos in client_id and qos, which must not be NULL.
- * Returns 1 on failure (m_d_s_s_s() not called, client_id or qos is NULL, sqlite error)
- * Returns 0 on success.
- */
-int mqtt3_db_sub_search_next(char **client_id, uint8_t *qos)
-{
-	/* Warning: Don't start transaction in this function. */
-	if(!stmt_sub_search || !client_id || !qos) return 1;
-	if(sqlite3_step(stmt_sub_search) != SQLITE_ROW){
-		return 1;
-	}
-	*client_id = mqtt3_strdup((char *)sqlite3_column_text(stmt_sub_search, 0));
-	*qos = sqlite3_column_int(stmt_sub_search, 1);
-
-	return 0;
-}
-
 /* Remove all subscriptions for a client.
  * Returns 1 on failure (client_id is NULL, sqlite error)
  * Returns 0 on success.
@@ -2018,8 +1993,10 @@ void mqtt3_db_sys_update(int interval, time_t start_time)
 			mqtt3_db_messages_easy_queue("", "$SYS/broker/clients/total", 2, strlen(buf), (uint8_t *)buf, 1);
 		}
 
+#ifdef WITH_MEMORY_TRACKING
 		snprintf(buf, 100, "%d", mqtt3_memory_used());
 		mqtt3_db_messages_easy_queue("", "$SYS/broker/heap/current size", 2, strlen(buf), (uint8_t *)buf, 1);
+#endif
 
 		snprintf(buf, 100, "%lu", mqtt3_net_msgs_total_received());
 		mqtt3_db_messages_easy_queue("", "$SYS/broker/messages/received", 2, strlen(buf), (uint8_t *)buf, 1);
@@ -2138,3 +2115,12 @@ void mqtt3_db_limits_set(int inflight, int queued)
 	max_queued = queued;
 }
 
+void mqtt3_db_vacuum(void)
+{
+	char *errmsg = NULL;
+	sqlite3_exec(db, "VACUUM", NULL, NULL, &errmsg);
+	if(errmsg){
+		sqlite3_free(errmsg);
+	}
+}
+
