@@ -29,17 +29,11 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include <config.h>
 
-#include <arpa/inet.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <ifaddrs.h>
 #include <netdb.h>
-#include <netinet/in.h>
-#include <stdint.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
-#include <sys/socket.h>
 #ifdef WITH_WRAP
 #include <tcpd.h>
 #endif
@@ -51,14 +45,16 @@ static uint64_t bytes_received = 0;
 static uint64_t bytes_sent = 0;
 static unsigned long msgs_received = 0;
 static unsigned long msgs_sent = 0;
-
-#ifdef WITH_CLIENT
-void (*client_net_write_callback)(int) = NULL;
-#endif
+static int max_connections = -1;
 
 static int _mqtt3_socket_listen(struct sockaddr *addr);
 
-int mqtt3_socket_accept(mqtt3_context **contexts, int *context_count, int listensock)
+void mqtt3_net_set_max_connections(int max)
+{
+	max_connections = max;
+}
+
+int mqtt3_socket_accept(mqtt3_context ***contexts, int *context_count, int listensock)
 {
 	int i;
 	int new_sock = -1;
@@ -71,6 +67,11 @@ int mqtt3_socket_accept(mqtt3_context **contexts, int *context_count, int listen
 
 	new_sock = accept(listensock, NULL, 0);
 	if(new_sock < 0) return -1;
+
+	if(max_connections > 0 && (*context_count) >= max_connections){
+		close(new_sock);
+		return -1;
+	}
 	/* Set non-blocking */
 	opt = fcntl(new_sock, F_GETFL, 0);
 	if(opt == -1 || fcntl(new_sock, F_SETFL, opt | O_NONBLOCK) == -1){
@@ -93,17 +94,17 @@ int mqtt3_socket_accept(mqtt3_context **contexts, int *context_count, int listen
 		if(!new_context) return -1;
 		mqtt3_log_printf(MQTT3_LOG_NOTICE, "New client connected from %s.", new_context->address);
 		for(i=0; i<(*context_count); i++){
-			if(contexts[i] == NULL){
-				contexts[i] = new_context;
+			if((*contexts)[i] == NULL){
+				(*contexts)[i] = new_context;
 				break;
 			}
 		}
 		if(i==(*context_count)){
 			(*context_count)++;
-			tmp_contexts = mqtt3_realloc(contexts, sizeof(mqtt3_context*)*(*context_count));
+			tmp_contexts = mqtt3_realloc(*contexts, sizeof(mqtt3_context*)*(*context_count));
 			if(tmp_contexts){
-				contexts = tmp_contexts;
-				contexts[(*context_count)-1] = new_context;
+				*contexts = tmp_contexts;
+				(*contexts)[(*context_count)-1] = new_context;
 			}
 		}
 #ifdef WITH_WRAP
@@ -122,7 +123,9 @@ int mqtt3_socket_close(mqtt3_context *context)
 
 	if(!context) return 1;
 	if(context->sock != -1){
-		mqtt3_db_client_invalidate_socket(context->id, context->sock);
+		if(context->id){
+			mqtt3_db_client_invalidate_socket(context->id, context->sock);
+		}
 		rc = close(context->sock);
 		context->sock = -1;
 	}
@@ -130,54 +133,6 @@ int mqtt3_socket_close(mqtt3_context *context)
 	return rc;
 }
 
-/* Create a socket and connect it to 'ip' on port 'port'.
- * Returns -1 on failure (ip is NULL, socket creation/connection error)
- * Returns sock number on success.
- */
-int mqtt3_socket_connect(const char *host, uint16_t port)
-{
-	int sock;
-	int opt;
-	struct addrinfo hints;
-	struct addrinfo *ainfo, *rp;
-	int s;
-
-	if(!host || !port) return -1;
-
-	memset(&hints, 0, sizeof(struct addrinfo));
-	hints.ai_family = AF_INET; /* IPv4 only at the moment. */
-	hints.ai_socktype = SOCK_STREAM;
-
-	s = getaddrinfo(host, NULL, &hints, &ainfo);
-	if(s) return -1;
-
-	for(rp = ainfo; rp != NULL; rp = rp->ai_next){
-		sock = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
-		if(sock == -1) continue;
-		
-		((struct sockaddr_in *)rp->ai_addr)->sin_port = htons(port);
-		if(connect(sock, rp->ai_addr, rp->ai_addrlen) != -1){
-			break;
-		}
-
-		return -1;
-	}
-	if(!rp){
-		mqtt3_log_printf(MQTT3_LOG_ERR, "Error: %s", strerror(errno));
-		return -1;
-	}
-	freeaddrinfo(ainfo);
-
-	/* Set non-blocking */
-	opt = fcntl(sock, F_GETFL, 0);
-	if(opt == -1 || fcntl(sock, F_SETFL, opt | O_NONBLOCK) == -1){
-		close(sock);
-		return -1;
-	}
-
-	return sock;
-}
-
 /* Internal function.
  * Create a socket and set it to listen based on the sockaddr details in addr.
  * Returns -1 on failure (addr is NULL, socket creation/listening error)
@@ -264,9 +219,9 @@ int mqtt3_socket_listen_if(const char *iface, uint16_t port)
 	return -1;
 }
 
-int mqtt3_net_packet_queue(mqtt3_context *context, struct _mqtt3_packet *packet)
+int mqtt3_net_packet_queue(mqtt3_context *context, struct _mosquitto_packet *packet)
 {
-	struct _mqtt3_packet *tail;
+	struct _mosquitto_packet *tail;
 
 	if(!context || !packet) return 1;
 
@@ -380,7 +335,7 @@ int mqtt3_net_read(mqtt3_context *context)
 	rc = mqtt3_packet_handle(context);
 
 	/* Free data and reset values */
-	mqtt3_context_packet_cleanup(&context->in_packet);
+	_mosquitto_packet_cleanup(&context->in_packet);
 
 	context->last_msg_in = time(NULL);
 	return rc;
@@ -390,7 +345,7 @@ int mqtt3_net_write(mqtt3_context *context)
 {
 	uint8_t byte;
 	ssize_t write_length;
-	struct _mqtt3_packet *packet;
+	struct _mosquitto_packet *packet;
 
 	if(!context || context->sock == -1) return 1;
 
@@ -405,9 +360,6 @@ int mqtt3_net_write(mqtt3_context *context)
 			write_length = write(context->sock, &packet->command, 1);
 			if(write_length == 1){
 				bytes_sent++;
-#ifdef WITH_CLIENT
-				packet->command_saved = packet->command;
-#endif
 				packet->command = 0;
 			}else{
 				if(write_length == 0) return 1; /* EOF */
@@ -465,14 +417,9 @@ int mqtt3_net_write(mqtt3_context *context)
 
 		msgs_sent++;
 
-#ifdef WITH_CLIENT
-		if(client_net_write_callback){
-			client_net_write_callback(packet->command_saved&0xF0);
-		}
-#endif
 		/* Free data and reset values */
 		context->out_packet = packet->next;
-		mqtt3_context_packet_cleanup(packet);
+		_mosquitto_packet_cleanup(packet);
 		mqtt3_free(packet);
 
 		context->last_msg_out = time(NULL);
@@ -480,101 +427,6 @@ int mqtt3_net_write(mqtt3_context *context)
 	return 0;
 }
 
-int mqtt3_read_byte(mqtt3_context *context, uint8_t *byte)
-{
-	if(context->in_packet.pos+1 > context->in_packet.remaining_length)
-		return 1;
-
-	*byte = context->in_packet.payload[context->in_packet.pos];
-	context->in_packet.pos++;
-
-	return 0;
-}
-
-int mqtt3_write_byte(struct _mqtt3_packet *packet, uint8_t byte)
-{
-	if(packet->pos+1 > packet->remaining_length) return 1;
-
-	packet->payload[packet->pos] = byte;
-	packet->pos++;
-
-	return 0;
-}
-
-int mqtt3_read_bytes(mqtt3_context *context, uint8_t *bytes, uint32_t count)
-{
-	if(context->in_packet.pos+count > context->in_packet.remaining_length)
-		return 1;
-
-	memcpy(bytes, &(context->in_packet.payload[context->in_packet.pos]), count);
-	context->in_packet.pos += count;
-
-	return 0;
-}
-
-int mqtt3_write_bytes(struct _mqtt3_packet *packet, const uint8_t *bytes, uint32_t count)
-{
-	if(packet->pos+count > packet->remaining_length) return 1;
-
-	memcpy(&(packet->payload[packet->pos]), bytes, count);
-	packet->pos += count;
-
-	return 0;
-}
-
-int mqtt3_read_string(mqtt3_context *context, char **str)
-{
-	uint16_t len;
-
-	if(mqtt3_read_uint16(context, &len)) return 1;
-
-	if(context->in_packet.pos+len > context->in_packet.remaining_length)
-		return 1;
-
-	*str = mqtt3_calloc(len+1, sizeof(char));
-	if(*str){
-		memcpy(*str, &(context->in_packet.payload[context->in_packet.pos]), len);
-		context->in_packet.pos += len;
-	}else{
-		return 1;
-	}
-
-	return 0;
-}
-
-int mqtt3_write_string(struct _mqtt3_packet *packet, const char *str, uint16_t length)
-{
-	if(mqtt3_write_uint16(packet, length)) return 1;
-	if(mqtt3_write_bytes(packet, (uint8_t *)str, length)) return 1;
-
-	return 0;
-}
-
-int mqtt3_read_uint16(mqtt3_context *context, uint16_t *word)
-{
-	uint8_t msb, lsb;
-
-	if(context->in_packet.pos+2 > context->in_packet.remaining_length)
-		return 1;
-
-	msb = context->in_packet.payload[context->in_packet.pos];
-	context->in_packet.pos++;
-	lsb = context->in_packet.payload[context->in_packet.pos];
-	context->in_packet.pos++;
-
-	*word = (msb<<8) + lsb;
-
-	return 0;
-}
-
-int mqtt3_write_uint16(struct _mqtt3_packet *packet, uint16_t word)
-{
-	if(mqtt3_write_byte(packet, MQTT_MSB(word))) return 1;
-	if(mqtt3_write_byte(packet, MQTT_LSB(word))) return 1;
-
-	return 0;
-}
-
 uint64_t mqtt3_net_bytes_total_received(void)
 {
 	return bytes_received;
