@@ -27,30 +27,19 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <arpa/inet.h>
-#include <errno.h>
-#include <netinet/in.h>
-#include <stdbool.h>
-#include <stdint.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <time.h>
-#include <unistd.h>
 
 #include <config.h>
 #include <mqtt3.h>
 
-int mqtt3_raw_connect(mqtt3_context *context, const char *client_id, bool will, uint8_t will_qos, bool will_retain, const char *will_topic, const char *will_msg, uint16_t keepalive, bool clean_start)
+int mqtt3_raw_connect(mqtt3_context *context, const char *client_id, bool will, uint8_t will_qos, bool will_retain, const char *will_topic, const char *will_msg, uint16_t keepalive, bool clean_session)
 {
-	struct _mqtt3_packet *packet = NULL;
+	struct _mosquitto_packet *packet = NULL;
 	int payloadlen;
 
 	if(!context || !client_id) return 1;
 
-	packet = mqtt3_calloc(1, sizeof(struct _mqtt3_packet));
+	packet = mqtt3_calloc(1, sizeof(struct _mosquitto_packet));
 	if(!packet) return 1;
 
 	payloadlen = 2+strlen(client_id);
@@ -69,16 +58,16 @@ int mqtt3_raw_connect(mqtt3_context *context, const char *client_id, bool will,
 	}
 
 	/* Variable header */
-	if(mqtt3_write_string(packet, PROTOCOL_NAME, strlen(PROTOCOL_NAME))) return 1;
-	if(mqtt3_write_byte(packet, PROTOCOL_VERSION)) return 1;
-	if(mqtt3_write_byte(packet, (will_retain<<5) | (will_qos<<3) | (will<<2) | (clean_start<<1))) return 1;
-	if(mqtt3_write_uint16(packet, keepalive)) return 1;
+	if(_mosquitto_write_string(packet, PROTOCOL_NAME, strlen(PROTOCOL_NAME))) return 1;
+	if(_mosquitto_write_byte(packet, PROTOCOL_VERSION)) return 1;
+	if(_mosquitto_write_byte(packet, ((will_retain&0x1)<<5) | ((will_qos&0x3)<<3) | ((will&0x1)<<2) | ((clean_session&0x1)<<1))) return 1;
+	if(_mosquitto_write_uint16(packet, keepalive)) return 1;
 
 	/* Payload */
-	if(mqtt3_write_string(packet, client_id, strlen(client_id))) return 1;
+	if(_mosquitto_write_string(packet, client_id, strlen(client_id))) return 1;
 	if(will){
-		if(mqtt3_write_string(packet, will_topic, strlen(will_topic))) return 1;
-		if(mqtt3_write_string(packet, will_msg, strlen(will_msg))) return 1;
+		if(_mosquitto_write_string(packet, will_topic, strlen(will_topic))) return 1;
+		if(_mosquitto_write_string(packet, will_msg, strlen(will_msg))) return 1;
 	}
 
 	context->keepalive = keepalive;
@@ -94,13 +83,13 @@ int mqtt3_raw_disconnect(mqtt3_context *context)
 int mqtt3_raw_subscribe(mqtt3_context *context, bool dup, const char *topic, uint8_t topic_qos)
 {
 	/* FIXME - only deals with a single topic */
-	struct _mqtt3_packet *packet = NULL;
+	struct _mosquitto_packet *packet = NULL;
 	uint32_t packetlen;
 	uint16_t mid;
 
 	if(!context || !topic) return 1;
 
-	packet = mqtt3_calloc(1, sizeof(struct _mqtt3_packet));
+	packet = mqtt3_calloc(1, sizeof(struct _mosquitto_packet));
 	if(!packet) return 1;
 
 	packetlen = 2 + 2+strlen(topic) + 1;
@@ -115,11 +104,11 @@ int mqtt3_raw_subscribe(mqtt3_context *context, bool dup, const char *topic, uin
 
 	/* Variable header */
 	mid = mqtt3_db_mid_generate(context->id);
-	if(mqtt3_write_uint16(packet, mid)) return 1;
+	if(_mosquitto_write_uint16(packet, mid)) return 1;
 
 	/* Payload */
-	if(mqtt3_write_string(packet, topic, strlen(topic))) return 1;
-	if(mqtt3_write_byte(packet, topic_qos)) return 1;
+	if(_mosquitto_write_string(packet, topic, strlen(topic))) return 1;
+	if(_mosquitto_write_byte(packet, topic_qos)) return 1;
 
 	if(mqtt3_net_packet_queue(context, packet)) return 1;
 	return 0;
@@ -129,13 +118,13 @@ int mqtt3_raw_subscribe(mqtt3_context *context, bool dup, const char *topic, uin
 int mqtt3_raw_unsubscribe(mqtt3_context *context, bool dup, const char *topic)
 {
 	/* FIXME - only deals with a single topic */
-	struct _mqtt3_packet *packet = NULL;
+	struct _mosquitto_packet *packet = NULL;
 	uint32_t packetlen;
 	uint16_t mid;
 
 	if(!context || !topic) return 1;
 
-	packet = mqtt3_calloc(1, sizeof(struct _mqtt3_packet));
+	packet = mqtt3_calloc(1, sizeof(struct _mosquitto_packet));
 	if(!packet) return 1;
 
 	packetlen = 2 + 2+strlen(topic);
@@ -150,10 +139,10 @@ int mqtt3_raw_unsubscribe(mqtt3_context *context, bool dup, const char *topic)
 
 	/* Variable header */
 	mid = mqtt3_db_mid_generate(context->id);
-	if(mqtt3_write_uint16(packet, mid)) return 1;
+	if(_mosquitto_write_uint16(packet, mid)) return 1;
 
 	/* Payload */
-	if(mqtt3_write_string(packet, topic, strlen(topic))) return 1;
+	if(_mosquitto_write_string(packet, topic, strlen(topic))) return 1;
 
 	if(mqtt3_net_packet_queue(context, packet)) return 1;
 	return 0;
