@@ -27,25 +27,11 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <errno.h>
-#include <stdbool.h>
-#include <stdint.h>
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/select.h>
-#include <unistd.h>
 
 #include <config.h>
 #include <mqtt3.h>
-
-#ifdef WITH_CLIENT
-/* Client callback for puback events - this WILL change. */
-void (*client_puback_callback)(int) = NULL;
-/* Client callback for pubcomp events - this WILL change. */
-void (*client_pubcomp_callback)(int) = NULL;
-#endif
+#include <util_mosq.h>
 
 int mqtt3_packet_handle(mqtt3_context *context)
 {
@@ -76,7 +62,7 @@ int mqtt3_packet_handle(mqtt3_context *context)
 		case UNSUBSCRIBE:
 			return mqtt3_handle_unsubscribe(context);
 #endif
-#if defined(WITH_CLIENT) || defined(WITH_BRIDGE)
+#ifdef WITH_BRIDGE
 		case CONNACK:
 			return mqtt3_handle_connack(context);
 		case SUBACK:
@@ -94,29 +80,32 @@ int mqtt3_handle_puback(mqtt3_context *context)
 {
 	uint16_t mid;
 
-	if(mqtt3_read_uint16(context, &mid)) return 1;
+	if(!context || context->in_packet.remaining_length != 2){
+		return 1;
+	}
+	if(_mosquitto_read_uint16(&context->in_packet, &mid)) return 1;
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received PUBACK from %s (Mid: %d)", context->id, mid);
 
-#ifdef WITH_CLIENT
-	if(client_puback_callback){
-		client_puback_callback(mid);
-	}
-#endif
-
 	if(mid){
-		if(mqtt3_db_message_delete(context->id, mid, md_out)) return 1;
+		if(mqtt3_db_message_delete(context->id, mid, mosq_md_out)) return 1;
 	}
 	return 0;
 }
 
 int mqtt3_handle_pingreq(mqtt3_context *context)
 {
+	if(!context || context->in_packet.remaining_length != 0){
+		return 1;
+	}
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received PINGREQ from %s", context->id);
 	return mqtt3_raw_pingresp(context);
 }
 
 int mqtt3_handle_pingresp(mqtt3_context *context)
 {
+	if(!context || context->in_packet.remaining_length != 0){
+		return 1;
+	}
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received PINGRESP from %s", context->id);
 	return 0;
 }
@@ -125,17 +114,14 @@ int mqtt3_handle_pubcomp(mqtt3_context *context)
 {
 	uint16_t mid;
 
-	if(mqtt3_read_uint16(context, &mid)) return 1;
+	if(!context || context->in_packet.remaining_length != 2){
+		return 1;
+	}
+	if(_mosquitto_read_uint16(&context->in_packet, &mid)) return 1;
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received PUBCOMP from %s (Mid: %d)", context->id, mid);
 
-#ifdef WITH_CLIENT
-	if(client_pubcomp_callback){
-		client_pubcomp_callback(mid);
-	}
-#endif
-
 	if(mid){
-		if(mqtt3_db_message_delete(context->id, mid, md_out)) return 1;
+		if(mqtt3_db_message_delete(context->id, mid, mosq_md_out)) return 1;
 	}
 	return 0;
 }
@@ -151,28 +137,28 @@ int mqtt3_handle_publish(mqtt3_context *context)
 	uint8_t header = context->in_packet.command;
 	int64_t store_id = 0;
 
-	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received PUBLISH from %s", context->id);
 	dup = (header & 0x08)>>3;
 	qos = (header & 0x06)>>1;
 	retain = (header & 0x01);
 
-	if(mqtt3_read_string(context, &topic)) return 1;
-	if(mqtt3_fix_sub_topic(&topic)) return 1;
+	if(_mosquitto_read_string(&context->in_packet, &topic)) return 1;
+	if(_mosquitto_fix_sub_topic(&topic)) return 1;
 	if(!strlen(topic)){
 		return 1;
 	}
 
 	if(qos > 0){
-		if(mqtt3_read_uint16(context, &mid)){
+		if(_mosquitto_read_uint16(&context->in_packet, &mid)){
 			mqtt3_free(topic);
 			return 1;
 		}
 	}
 
 	payloadlen = context->in_packet.remaining_length - context->in_packet.pos;
+	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received PUBLISH from %s (%d, %d, %d, %d, '%s', ... (%ld bytes))", context->id, dup, qos, retain, mid, topic, (long)payloadlen);
 	if(payloadlen){
 		payload = mqtt3_calloc(payloadlen+1, sizeof(uint8_t));
-		if(mqtt3_read_bytes(context, payload, payloadlen)){
+		if(_mosquitto_read_bytes(&context->in_packet, payload, payloadlen)){
 			mqtt3_free(topic);
 			return 1;
 		}
@@ -198,7 +184,7 @@ int mqtt3_handle_publish(mqtt3_context *context)
 			if(mqtt3_raw_puback(context, mid)) rc = 1;
 			break;
 		case 2:
-			if(mqtt3_db_message_insert(context->id, mid, md_in, ms_wait_pubrec, qos, store_id)) rc = 1;
+			if(mqtt3_db_message_insert(context->id, mid, mosq_md_in, ms_wait_pubrec, qos, store_id)) rc = 1;
 			if(mqtt3_raw_pubrec(context, mid)) rc = 1;
 			break;
 	}
@@ -212,10 +198,13 @@ int mqtt3_handle_pubrec(mqtt3_context *context)
 {
 	uint16_t mid;
 
-	if(mqtt3_read_uint16(context, &mid)) return 1;
+	if(!context || context->in_packet.remaining_length != 2){
+		return 1;
+	}
+	if(_mosquitto_read_uint16(&context->in_packet, &mid)) return 1;
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received PUBREC from %s (Mid: %d)", context->id, mid);
 
-	if(mqtt3_db_message_update(context->id, mid, md_out, ms_wait_pubcomp)) return 1;
+	if(mqtt3_db_message_update(context->id, mid, mosq_md_out, ms_wait_pubcomp)) return 1;
 	if(mqtt3_raw_pubrel(context, mid)) return 1;
 
 	return 0;
@@ -225,10 +214,13 @@ int mqtt3_handle_pubrel(mqtt3_context *context)
 {
 	uint16_t mid;
 
-	if(mqtt3_read_uint16(context, &mid)) return 1;
+	if(!context || context->in_packet.remaining_length != 2){
+		return 1;
+	}
+	if(_mosquitto_read_uint16(&context->in_packet, &mid)) return 1;
 	mqtt3_log_printf(MQTT3_LOG_DEBUG, "Received PUBREL from %s (Mid: %d)", context->id, mid);
 
-	if(mqtt3_db_message_release(context->id, mid, md_in)) return 1;
+	if(mqtt3_db_message_release(context->id, mid, mosq_md_in)) return 1;
 	if(mqtt3_raw_pubcomp(context, mid)) return 1;
 
 	return 0;
